//  -*- Mode: C++ -*-

// collections.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2003      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class StellaHashTable : public AbstractHashTable {
public:
  KvCons** theTable;
  int size;
  int initialSize;
  int freeElements;
public:
  virtual void removeAt(Object* key);
  virtual void insertAt(Object* key, Object* value);
  virtual Object* lookup(Object* key);
  virtual Surrogate* primaryType();
};


// Global declarations:
extern boolean oUSE_STELLA_HASH_TABLESpo;
extern double oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo;

// Function signatures:
boolean stellaCollectionP(Object* self);
boolean terminateSetP(Set* self);
boolean terminateSetIteratorP(SetIterator* self);
Vector* stella::vector(int values, ...);
boolean vectorNextP(AllPurposeIterator* self);
StellaHashTable* newStellaHashTable();
Object* accessStellaHashTableSlotValue(StellaHashTable* self, Symbol* slotname, Object* value, boolean setvalueP);
void initializeStellaHashTable(StellaHashTable* self);
StellaHashTable* makeStellaHashTable(int initialsize);
void rehashStellaHashTable(StellaHashTable* self, int newsize);
void stellaHashTableInsertAt(StellaHashTable* self, Object* key, Object* value);
void stellaHashTableRemoveAt(StellaHashTable* self, Object* key);
Object* stellaHashTableLookup(StellaHashTable* self, Object* key);
Object* stellaStringHashTableLookup(StellaHashTable* self, char* key);
void helpStartupCollections1();
void helpStartupCollections2();
void startupCollections();

// Auxiliary global declarations:
extern Surrogate* SGT_COLLECTIONS_STELLA_COLLECTION;
extern Symbol* SYM_COLLECTIONS_STELLA_VECTOR;
extern Keyword* KWD_COLLECTIONS_CPP;
extern Keyword* KWD_COLLECTIONS_FUNCTION;
extern Surrogate* SGT_COLLECTIONS_STELLA_VECTOR;
extern Surrogate* SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE;
extern Symbol* SYM_COLLECTIONS_STELLA_SIZE;
extern Symbol* SYM_COLLECTIONS_STELLA_INITIAL_SIZE;
extern Symbol* SYM_COLLECTIONS_STELLA_FREE_ELEMENTS;
extern Symbol* SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS;
extern Symbol* SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
